@@ -25,6 +25,7 @@ class NodeBase(object):
     def __init__(self, nodegraph, _id):
         self._nodegraph = nodegraph
         self._id = _id
+        self._idname = None
         self._pos = wx.Point(0, 0)
         self._size = wx.Size(NODE_DEFAULT_WIDTH, NODE_DEFAULT_HEIGHT)
         self._selected = False
@@ -38,10 +39,11 @@ class NodeBase(object):
         self._category = "INPUT"
         self._headercolor = "#fff"
 
-    def _Init(self):
+    def _Init(self, idname):
         self.InitSockets()
         self.InitHeaderColor()
-        
+        self.SetIdName(idname)
+
     @property
     def nodegraph(self):
         return self._nodegraph
@@ -91,6 +93,9 @@ class NodeBase(object):
             if socket.HitTest(pos - self.pos):
                 return socket
 
+    def EditParameter(self, idname, value):
+        pass
+
     def IsOutputNode(self):
         return self._isoutput
 
@@ -110,7 +115,7 @@ class NodeBase(object):
         for param in self._parameters:
             ins.append((param, "RENDERIMAGE"))
 
-        if self.IsOutputNode() != True:
+        if self.IsOutputNode() is not True:
             outs = [('Output', "RENDERIMAGE")]
 
         x, y = self.pos
@@ -132,10 +137,15 @@ class NodeBase(object):
 
         self._sockets = sockets
 
-        # Adjust the size of the node to fit 
+        # Adjust the size of the node to fit
         # the amount of sockets the node has.
         self.size[1] = lastcoord
 
+    def GetIdname(self):
+        return self._idname
+
+    def SetIdName(self, idname):
+        self._idname = idname
 
     def GetPosition(self):
         return self.pos
@@ -161,6 +171,9 @@ class NodeBase(object):
     def SetActive(self, active=True):
         self.active = active
 
+    def GetSockets(self):
+        return self._sockets
+
     def Draw(self, dc):
         x, y = self.GetPosition()
         w, h = self.GetSize()
